@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( uninitialized once );
 
-our $VERSION = '1.879';
+our $VERSION = '1.901';
 
 use base 'MCE::Channel';
 use MCE::Mutex ();
@@ -217,19 +217,22 @@ sub recv_nb {
 sub send2 {
    my $self = shift;
    my $data = $freeze->([ @_ ]);
-
-   local $\ = undef if (defined $\);
-   local $MCE::Signal::SIG;
+   my $sig;
 
    {
-      local $MCE::Signal::IPC = 1;
-      ( my $c_mutex = $self->{c_mutex} )->lock2;
+      local $SIG{ABRT} = local $SIG{HUP} =
+      local $SIG{QUIT} = local $SIG{INT} =
+      local $SIG{TERM} = sub {
+         $sig = $_[0];
+      };
 
-      print { $self->{c_sock} } pack('i', length $data), $data;
-      $c_mutex->unlock2;
+      local $\ = undef if (defined $\);
+      $self->{c_mutex}->synchronize2( sub {
+         print { $self->{c_sock} } pack('i', length $data), $data;
+      });
    }
 
-   CORE::kill($MCE::Signal::SIG, $$) if $MCE::Signal::SIG;
+   CORE::kill($sig, $$) if $sig;
 
    return 1;
 }
@@ -304,7 +307,7 @@ MCE::Channel::Mutex - Channel for producer(s) and many consumers
 
 =head1 VERSION
 
-This document describes MCE::Channel::Mutex version 1.879
+This document describes MCE::Channel::Mutex version 1.901
 
 =head1 DESCRIPTION
 
@@ -323,7 +326,7 @@ The API is described in L<MCE::Channel>.
  my $chnl_a = MCE::Channel->new( impl => 'Mutex' );
 
  # Specify the 'mp' option for safe use by two or more producers
- # sending or recieving on the left side of the channel (i.e.
+ # sending or receiving on the left side of the channel (i.e.
  # ->enqueue/->send or ->recv2/->recv2_nb).
 
  my $chnl_b = MCE::Channel->new( impl => 'Mutex', mp => 1 );
