@@ -11,11 +11,12 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized once );
 
-our $VERSION = '1.877';
+our $VERSION = '1.901';
 
 use base 'MCE::Mutex';
 use Fcntl ':flock';
-use Carp ();
+use Scalar::Util 'looks_like_number';
+use Time::HiRes 'alarm';
 
 my $tid = $INC{'threads.pm'} ? threads->tid() : 0;
 
@@ -23,11 +24,16 @@ sub CLONE {
     $tid = threads->tid() if $INC{'threads.pm'};
 }
 
+sub MCE::Mutex::Flock::_guard::DESTROY {
+    my ($pid, $obj) = @{ $_[0] };
+    CORE::flock ($obj->{_fh}, LOCK_UN), $obj->{ $pid } = 0 if $obj->{ $pid };
+
+    return;
+}
+
 sub DESTROY {
     my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, @_);
-
     $obj->unlock(), close(delete $obj->{_fh}) if $obj->{ $pid };
-
     unlink $obj->{path} if ($obj->{_init} && $obj->{_init} eq $pid);
 
     return;
@@ -123,6 +129,11 @@ sub lock {
     return;
 }
 
+sub guard_lock {
+    &lock(@_);
+    bless([ $tid ? $$ .'.'. $tid : $$, $_[0] ], MCE::Mutex::Flock::_guard::);
+}
+
 *lock_exclusive = \&lock;
 
 sub lock_shared {
@@ -153,20 +164,37 @@ sub synchronize {
     $obj->_open() unless exists $obj->{ $pid };
 
     # lock, run, unlock - inlined for performance
-    CORE::flock ($obj->{_fh}, LOCK_EX), $obj->{ $pid } = 1
-        unless $obj->{ $pid };
-
+    my $guard = bless([ $pid, $obj ], MCE::Mutex::Flock::_guard::);
+    unless ($obj->{ $pid }) {
+        CORE::flock ($obj->{_fh}, LOCK_EX), $obj->{ $pid } = 1;
+    }
     (defined wantarray)
       ? @ret = wantarray ? $code->(@_) : scalar $code->(@_)
       : $code->(@_);
 
-    CORE::flock ($obj->{_fh}, LOCK_UN), $obj->{ $pid } = 0;
-
     return wantarray ? @ret : $ret[-1];
 }
 
 *enter = \&synchronize;
 
+sub timedwait {
+    my ($obj, $timeout) = @_;
+    die 'MCE::Mutex::Flock::timedwait() unimplemented in this platform'
+        if ($^O eq 'MSWin32');
+
+    $timeout = 1 unless defined $timeout;
+    Carp::croak('MCE::Mutex: timedwait (timeout) is not valid')
+        if (!looks_like_number($timeout) || $timeout < 0);
+
+    $timeout = 0.0003 if $timeout < 0.0003;
+
+    local $@; local $SIG{ALRM} = sub { alarm 0; die "timed out\n" };
+    eval { alarm $timeout; $obj->lock_exclusive };
+    alarm 0;
+
+    ( $@ && $@ eq "timed out\n" ) ? '' : 1;
+}
+
 1;
 
 __END__
@@ -183,7 +211,7 @@ MCE::Mutex::Flock - Mutex locking via Fcntl
 
 =head1 VERSION
 
-This document describes MCE::Mutex::Flock version 1.877
+This document describes MCE::Mutex::Flock version 1.901
 
 =head1 DESCRIPTION
 
@@ -201,6 +229,8 @@ The API is described in L<MCE::Mutex>.
 
 =item lock_shared
 
+=item guard_lock
+
 =item unlock
 
 =item synchronize
